% Este script serve como exemplo para aplicaes de condies de fronteira
% de Robin no clssico exemplo de crculo unitrio, sendo que o problema 
% resolvido com elementos triangulares de 6 ns.

clear
clc
close all

% Posies dos ns e malhas definidas manualmente. 13 ns e 4 elementos.
% Primeiros 8 ns representam a "cirunferncia" exterior.
x = [1 0.7071 0 -0.7071 -1 -0.7071 0 0.7071 0.5 0 -0.5 0 0]';
y = [0 0.7071 1 0.7071 0 -0.7071 -1 -0.7071 0 0.5 0 -0.5 0]';
tri6 = [1 3 13 2 10 9;5 13 3 11 10 4;5 7 13 6 12 11;7 1 13 8 9 12];

nEl = size(tri6,1); % Nmero de elementos = nmero de linhas na tabela de conectividades.
nDOFS = size(x,1); % Nmero de ns = nmero de x's. (ou y's)

% Plot da malha inicial para visualizao.

figure
plot(x,y,'ro'); hold on
for i = 1:nEl
    DOFS = [tri6(i,1) tri6(i,4) tri6(i,2) ...
        tri6(i,5) tri6(i,3) tri6(i,6) tri6(i,1)]; % 6 ns + o primeiro para "fechar"
    plot(x(DOFS),y(DOFS),'b');
end
grid on
hold off

% Pr-alocao de matrizes globais.
Kg = zeros(nDOFS,nDOFS);
fg = zeros(nDOFS,1);

%% Assemblagem
for i = 1:nEl
    DOFS = tri6(i,:);
    posDOFS(1,1:6) = x(DOFS);
    posDOFS(2,1:6) = y(DOFS);
    
    % Clculo das matrizes elementares
    f = 4;
    [Ke, fe] = ElemTri6(posDOFS,f);
    
    Kg(DOFS,DOFS) = Kg(DOFS,DOFS) + Ke;
    fg(DOFS,1)    = fg(DOFS,1)    + fe;
end

%% Aplicao de condies de fronteira naturais atravs da funo RobinQuadratico
p = 100;
gama = 2;
% So 4 os elementos na fronteira. Tambm se podia aplicar individualmente
% a cada um deles as CFs.
for i = 1:4
    DOF1 = 2*i - 1;
    DOF2 = 2*i - 1 + 1;
    if i ~= 4
        DOF3 = 2*i - 1 + 2;
    else
        % Para o ltimo elemento  necessrio ligar ao primeiro n.
        DOF3 = 1;
    end
    
    DOFS = [DOF1, DOF2, DOF3];
    [He,pe] = RobinQuadratico(x(DOF1),y(DOF1),x(DOF2),y(DOF2),...
        x(DOF3),y(DOF3),p,gama);
    
    Kg(DOFS,DOFS) = Kg(DOFS,DOFS) + He;
    fg(DOFS,1)    = fg(DOFS,1)    + pe;
end


%% Resoluo dos sistema
Kg = sparse(Kg);
fg = sparse(fg);
u = Kg\fg;

%% Exemplo da representao da soluo com fill.

figure
for i = 1:nEl
 DOFS = [tri6(i,1) tri6(i,4) tri6(i,2) ...
        tri6(i,5) tri6(i,3) tri6(i,6) tri6(i,1)]; 
   fill(x(DOFS),y(DOFS),u(DOFS));
    hold on
   plot(x(DOFS),y(DOFS),'b');
end
plot(x,y,'ro');
grid on
colorbar;
