%% 
% Funo que devolve as matrizes He e pe,tendo em conta a aplicao de
% condies de fronteira naturais num elemento quadrtico, tendo como
% entrada as posies dos trs ns e os valores de p e gama.

function [He,pe] = RobinQuadratico(x1,y1,x2,y2,x3,y3,p,gama)

% Pr-alocao das matrizes He e pe.
He = zeros(3,3);
pe = zeros(3,1);

% Obter o nmero de pontos de integrao desejados e respectivos pesos.
n = 2; % at n = 4
[x,w] = GaussLegendre1D(n);

for i = 1:n
    % Para cada ponto de integrao, calcula-se a contribuio para He e pe.
    xi = x(i);
    
    % Funes de forma para o elemento 1D quadrtico.
    psi(1) = 1/2 * xi*(xi-1);
    psi(2) = 1 - xi^2;
    psi(3) = 1/2 * xi*(xi+1);
    
    % Derivadas das funes de forma para o mesmo elemento.
    dpsi(1) = xi - 1/2;
    dpsi(2) = -2*xi;
    dpsi(3) = xi + 1/2;
    
    % Cculo de dx/dxi e dy/dxi para o clculo do "determinante" da matriz
    % jacobiana. 
    dxdxi = dpsi(1)*x1 + dpsi(2)*x2 + dpsi(3)*x3;
    dydxi = dpsi(1)*y1 + dpsi(2)*y2 + dpsi(3)*y3;
    dsdxi = sqrt(dxdxi^2 + dydxi^2);
    
    % Somatrio para as matrizes He e pe.
    He = He + psi'*psi*dsdxi*w(i);
    pe = pe + (psi*dsdxi*w(i))';
    
end
% Admitindo que p e gama so constantes.
He = p*He;
pe = gama*pe;

end